# Fax2 API

import sys
import os
import time
from urllib import urlencode
import urllib2
import json

BASE_URL = "https://api.fax2.com.au/v1/"

def time_as_iso8601(t):
	return time.strftime("%Y-%m-%dT%H:%M:%SZ",time.gmtime(t))

def time_from_iso8601(s):
	return time.mktime(time.strptime(s,"%Y-%m-%dT%H:%M:%SZ"))

class TokenAuthHandler(urllib2.BaseHandler):
	def __init__(self,apiobj):
		self.apiobj = apiobj

	def http_request(self,request):
		token = self.apiobj.auth_token
		request.add_header("Authorization","bearer "+token)
		return request
		
	https_request = http_request
	
class Fax2Error(Exception):
	def __init__(self,data):
		print data
		self.code = data['error']
		self.description = None
		if 'error_description' in data:
			self.description = data['error_description']
		self.url = None
		if 'more_info' in data:
			self.url = data['more_info']
			
	def message(self):
		return self.code
	
	def __str__(self):
		s = "Fax2 API Error: %s" % (self.code)
		if self.description:
			s = s + " (%s)" % (self.description)
		if self.url:
			s = s + " (see %s)" % (self.url)
		return s

class Fax2API(object):
	def __init__(self):
		self.auth_token_expires = 0
		token_handler = TokenAuthHandler(self)
		self.opener = urllib2.build_opener(token_handler)
	
	def credentials(self,username,password):
		self.username = username
		self.password = password

	def get_auth_token(self):
		auth_handler = urllib2.HTTPBasicAuthHandler()
		auth_handler.add_password('Fax2 API',BASE_URL,self.username,self.password)
		opener = urllib2.build_opener(auth_handler)
		req = urllib2.Request(
			BASE_URL+'oauth2/token',
			urlencode({'grant_type':'client_credentials'})
		)
		resp = opener.open(req)
		data = json.loads(resp.read())
		self.auth_token = data['access_token']
		self.auth_token_expires = time.time() + data['expires_in']

	def request_url(self,url,*args):
		if self.auth_token_expires - time.time() < 60:
			self.get_auth_token()
	
		req = urllib2.Request(url,*args)
		try:
			return self.opener.open(req)
		except urllib2.HTTPError as e:
			if e.code == 400:
				data = json.loads(e.read())
				raise Fax2Error,data,sys.exc_info()[2]
			else:
				raise

	def request(self,endpoint,*args):
		return self.request_url(BASE_URL+endpoint,*args)

	def json_request(self,*args):
		resp = self.request(*args)
		return json.loads(resp.read())
		
	def json_request_url(self,*args):
		resp = self.request_url(*args)
		return json.loads(resp.read())
		
	def received_faxes(self,from_time=None,before_time=None,limit=None):
		args = {}
		
		if from_time:
			args['from_time'] = time_as_iso8601(from_time)
		if before_time:
			args['before_time'] = time_as_iso8601(before_time)
		if limit:
			args['limit'] = str(limit)
			
		data = []
			
		resp = self.json_request("received_faxes?" + urlencode(args))
		while('next_page_url' in resp):
			data = data + resp['data']
			resp = self.json_request_url(resp['next_page_url'])
		data = data + resp['data']
		
		for entry in data:
			entry['received_at'] = time_from_iso8601(entry['received_at'])
		
		return data

	def get_fax_pdf_data(self,faxid):
		resp = self.request("received_faxes/%s/content.pdf" % (faxid))
		return resp.read()
	
	def write_fax_pdf_data(self,faxid,filename):
		data = self.get_fax_pdf_data(faxid)
		tempname = "%s.%d" % (filename,os.getpid())
		f = open(tempname,"wb")
		f.write(data)
		f.close()
		os.rename(tempname,filename)

	def upload_document(self,document_data,mime_type):
		data = self.json_request(
			"upload_document",
			document_data,
			{"Content-Type":mime_type}
		)
		return data['document_id']
	
	def send_fax(self,dest_number,documents):
		data = self.json_request(
			"send_fax",
			urlencode({"documents[]":documents,"dest_number":dest_number},True)
		)
		return (data['id'],data['pages'])
		
	def get_outgoing_fax_status(self,fax_id):
		data = self.json_request("sent_faxes/%s" % (fax_id))
		return data
